title: How “Good Intent” Undermines Diversity and Inclusion
url: https://thebias.com/2017/09/26/how-good-intent-undermines-diversity-and-inclusion/
hash_url: 618f913d970fee8feadadd15cf282e5a

<p>A lot of codes of conduct, community guidelines, and company values statements ask people to “assume good intent” when in conflict with other members. Positive statements like this feel more pleasant and welcoming than lists of banned behavior. But asking people to “assume good intent” will actually undermine your code of conduct and make marginalized people feel less welcome and less safe in your community.</p>
<p>A positive rule is still a rule, and it still bans behavior that contradicts it. You need to think carefully about what behaviors your positive expectations are banning, and who those bans will affect. “Assume good intent” is a particularly pernicious positive expectation that will undermine your code of conduct. The implied inverse of this is that <i>not</i> assuming good intent is against the rules.</p>
<p>On its face, that might not sound like a bad idea. After all, isn’t assuming the best in others generally a good way to go through life? What’s the harm in encouraging that within your community?</p>
<p>The harm is that telling people to “assume good intent” is a sign that if they come to you with a concern, you will minimize their feelings, police their reactions, and question their perceptions. It tells marginalized people that you don’t see codes of conduct as tools to address systemic discrimination, but as tools to manage personal conflicts without taking power differences into account. Telling people to “assume good intent” sends a message about whose feelings you plan to center when an issue arises in your community.</p>
<h2>Codes of Conduct Address Systemic Inequities</h2>
<p>Codes of conduct have become standard in geek spaces because people have become more aware of the ways that our spaces are unsafe and unwelcoming to people of color of all genders, women of all races, and other marginalized people. They’re a tool that helps us build safer, more welcoming, and more inclusive spaces. It’s important to keep that goal in mind.</p>
<p>It’s tempting to lead with positivity and encourage a friendly atmosphere, but we don’t end harassment and discrimination just by telling people to be nice. Even someone with no bad intentions can still cause harm if they’re being ignorant or careless. A classic example is stepping on someone’s foot: whether you mean to do it or not doesn’t change the fact that stepping on somebody’s foot hurts.</p>
<p>The strength of this example is that it removes privilege and systemic discrimination from the equation, which makes it much easier for people who don’t understand institutional power to grasp. But trying to explain the impact of racism, sexism, and other forms of systemic discrimination without addressing them as a system is impossible.</p>
<p>Talking about stepping on someone’s foot as a stand-in for micro-aggressions still leaves people confused. Okay, sure, if you step on someone’s foot, you move and apologize–but if it was an accident, then even if it hurt, it doesn’t justify the person cussing you out or shoving you off their foot, right? Shouldn’t civility go both ways? Shouldn’t people assume good intent and ask politely for the other person to move?</p>
<p>So I am not going to ask you to put yourself in the shoes of someone who’s inadvertently stepped on someone’s foot. Instead, I’m going to ask you to imagine that your foot’s been stepped on.</p>
<p>But not just once.</p>
<p>Your foot has been stepped on <i>every single day of your life</i>.</p>
<p>A few people have done it on purpose, but most of the time, it’s been an accident. The people who do it don’t hate you. Most of them don’t even know you. Some of them are your friends. Some of the people who’ve stepped on your foot genuinely love you.</p>
<p>And yet, every day, your foot is getting stepped on. Maybe it’s because of your race, or your gender, or because you’re disabled. Maybe it’s because you’re fat or poor. Maybe all of the above. Point is, people are constantly stepping on you.</p>
<p>You learn to stand back. To yield space to people who might hurt you. To give other people the right of way when walking in crowds, so they won’t walk right into you and <i>BAM</i> bring their heel down on your good dress shoes. You’re constantly being told that you need to watch where you put your feet.</p>
<p>How long, do you think, would you put up with that before you’d start wondering why people are telling you to watch your feet instead of telling the people who step on you to watch theirs? How long would it take you to stop caring whether or not people <i>mean</i> to do it? Because when push comes to stomp, they clearly don’t mean <i>not</i> to do it, or it wouldn’t. keep. happening.</p>
<p>In that context, people telling you to ‘assume good intent’ sounds like they’re really telling you to shut up. That your feelings about getting stomped on all the time don’t matter. That no matter how sore your foot is, how much money you’ve spent replacing ruined shoes, how many times you’ve limped on broken toes, you still have a responsibility to worry about the feelings of the people who are hurting you. Because they don’t <i>mean</i> it. As if that makes a difference.</p>
<p>As a community leader, you don’t want to build spaces where people react calmly to getting their foot stepped on for the millionth time. You want to build spaces where people can trust that they are safe from being stepped on. To do that, you need to address the <i>system</i> of behavior that makes marginalized people feel unwelcome, rather than treating each instance of that behavior as a personal conflict that has occurred in isolation.</p>
<p>This isn’t to say that intent doesn’t matter at all. Certainly if you know a person <i>did</i> mean to violate your code of conduct, it’s appropriate to take that into account when deciding how to respond. You don’t have a quiet word about manners with someone who intentionally used a racial slur; you show them them the door.</p>
<p>But when someone was being careless instead of malicious, their carelessness doesn’t erase the harm. You need to address that harm by centering the <i>victim’s</i> feelings, instead of asking them to center the feelings of the person who hurt them. You need to be sensitive to the fact that the individual incident is part of a larger pattern of behavior they experience, and take steps to keep that pattern of behavior out of your space.</p>
<h2>The False Equivalence of Treating Harassment as Interpersonal Conflict</h2>
<p>Being mindful of systemic inequalities is essential when handling code of conduct concerns. Addressing incidents as if they’re simple conflicts between the parties involved sets up a false equivalence between dealing with discrimination and dealing with the momentary discomfort of being told you hurt someone.</p>
<p>Imagine you’re a community leader. A member of your community, Fred, comes to you and tells you Alicia just cussed him out and he’s upset. You go and talk to Alicia. It turns out, Fred stepped on Alicia’s foot. Alicia, shocked and in pain, shouted “Ow, Fred, <i>what the fuck</i>!?”</p>
<p>Fred insists this wasn’t fair, and Alicia owes him an apology, because he didn’t mean to step on her foot, and she made him feel bad. Your code of conduct says people should “assume good intent,” and Fred is lodging a complaint that Alicia failed to do that. He refuses to apologize for stepping on her foot until she apologizes for cursing.</p>
<p>(Round about now, you might be thinking this is an absurd hypothetical for any community other than a kindergarten classroom. I’ve been consulting on community safety for five years, and friend, I am here to tell you: <i>I’m toning this example</i> way <i>down</i>.)</p>
<p>Back to Alicia and Fred.</p>
<p>Alicia may have a broken toe. Her only pair of dress shoes may be ruined. But the question of whether Fred owes her an apology for this has been completely derailed while you entertain the false equivalence that Alicia dropping an f-bomb has harmed him as much as his negligence harmed her.</p>
<h2>Invalidating Victims’ Feelings</h2>
<p>Worrying about ‘assuming good intent’ when one party has harmed another is centering the feelings of the person who behaved badly, and expecting the person they hurt to center their feelings, too.</p>
<p>Look at the situation between Fred and Alicia. Can you imagine telling Alicia that she and Fred are both in the wrong? Can you imagine telling her that she has a responsibility to consider Fred’s feelings, when Fred is showing no concern for hers?</p>
<p>Telling people to ‘assume good intent’ is telling them that no matter how badly they hurt, they still need to smile and be nice so the person who hurt them won’t feel blamed.</p>
<p>This creates a double standard. Alicia must assume good intent from Fred, even if he stepped on her foot because he was helping himself to her personal space in a way he would never do to another man. But when Alicia reacts out of shock, anger, and pain, the ‘assume good intent’ rule allows Fred to cast that as something Alicia has done <i>at</i> him, rather than seeing it as a very normal human response to being hurt.</p>
<h2>Policing Victims’ Reactions</h2>
<p>Including “assume good intent” in your code of conduct tells victims that they aren’t safe in your space, because if they do anything to make others feel bad about harming them, they will be held accountable for breaking the rules. </p>
<p>In Fred and Alicia’s case, Fred can hide behind “assume good intent” to say that his intentions absolve him of responsibility for hurting Alicia. At the same time, he can demand Alicia take responsibility for making him feel bad. And he can do this no matter how she reacted. Even if she didn’t cuss at him, if he in any way feels blamed or disrespected by how she tells him to get off her foot, he can accuse her of breaking the rules by not ‘assuming good intent.’</p>
<p>“Assume good intent” is frequently brought into codes of conduct to attempt to create a culture of blamelessness, but in reality, it places the question of blame front and center. If Fred didn’t mean any harm, he should be willing to accept responsibility, instead of insisting he’s not to blame. He should understand that your first concern should be making the space safe and welcoming for Alicia and other members of marginalized groups.</p>
<h2>Questioning Victims’ Perceptions</h2>
<p>Marginalized people already know that we’re supposed to “assume good intent” in others. We are told every day that we’re “paranoid,” “overreacting,” or just plain “crazy” if we don’t feel good about being treated badly. This process is called ‘gaslighting,’ and it’s a way of making marginalized people distrust our own perceptions so we won’t object to being mistreated.</p>
<p>In <i>The Gift of Fear</i>, Gavin de Becker talks a lot about instinct, and the way that women develop ‘gut feelings’ about men who are trying to harm them. The central thesis of the book is that women should learn to trust these instincts because they’re based on concrete observations of dangerous behavior. They are a form of pattern recognition that women develop from years of experience. Members of other marginalized groups develop similar forms of pattern recognition to protect themselves from harm, often based on signs so small they can’t consciously describe them.</p>
<p>When you tell people in your community to “assume good intent,” you’re reinforcing the notion that marginalized people shouldn’t trust their instincts.</p>
<p>If Alicia is angry at Fred, it’s not because she’s being vindictive or irrational. Fred has shown Alicia that he thinks his feelings are more important than hers. He thinks Alicia owes it to him to be nice and make him feel good about himself even when he hurt her. He thinks this so strongly that he won’t treat her with basic human decency until she ‘earns it’ by apologizing to him. He has shown Alicia that he thinks it’s okay to harm her if she’s not nice to him.</p>
<p>Fred is dangerous. It would be completely irrational for Alicia to extend him the benefit of the doubt. Forcing her to do so would make your community less safe and inclusive, not more.</p>
<p>Just like it would make your community less safe and inclusive to require that other marginalized members of your community ‘assume good intent’ in those who have demonstrated quite clearly that their intent isn’t good.</p>
<h2>Who Are You Protecting?</h2>
<p>People often reach for positive statements like “assume good intent” because they’re worried about people being “shamed” over innocent mistakes. But society at large is already inclined to assume good intent in people with power and privilege–even when they’re not demonstrating it. If you want to build a culture of “assuming good intent,” start by assuming good intent in marginalized people.</p>
<p>Assume that they already tried being nice. Assume that their feelings are valid. Assume that, after a lifetime of practice, they are responding to harmful behavior in the way that is safest for them. Prioritize that safety over the momentary discomfort people feel when they realize they’ve done something hurtful.</p>
<p>Culture-setting documents like your code of conduct and corporate values should be designed around protecting marginalized people from harmful behavior. Leave out “assume good intent.” Instead, create a culture that recognizes and pushes back against the ways that marginalized people are dehumanized. Expect people to demonstrate their good intent by treating people with respect.</p>
