title: Variability, Not Repetition, is the Key to Mastery
url: https://www.scotthyoung.com/blog/2022/10/26/variable-mastery/
hash_url: 3d3259256af967c84b199220b3a5244a

<p>Bruce Lee is reported to have said, “I fear not the man who has practiced 10,000 kicks once, but the man who has practiced one kick 10,000 times.” With all due respect to Mr. Lee, he might have been wrong about this one.</p>
<p>Variability plays an essential and oft-neglected role in mastering complex skills. Considerable research shows that practicing in varied contexts with varied methods and performing with varied task constraints results in more robust learning than simple repetition.</p>
<p>Below, I’d like to review some of the key research supporting the role of variability in learning and suggest how you can apply this to your career and studies.</p>
<h2>Contextual Interference: Same Method, Variable Situations</h2>
<p class="inline-podcast">
<small>Listen to this article</small><br>
</p>
<p><a href="https://en.wikipedia.org/wiki/Motor_learning#Structure_of_practice_and_contextual_interference">Contextual interference</a> occurs when you practice the same skill, but vary the situations in which it is called for.</p>
<p>For instance, you could practice your tennis backhand by being served backhand shots repeatedly. Alternatively, your coach could mix things up: serve you backhand shots interspersed with balls that require a forehand shot.</p>
<p>Or imagine preparing for a calculus exam: you could study all the questions that require the chain rule, then all the questions that use the quotient rule. Instead, you might shuffle these questions together so you can’t be sure which technique is needed.</p>
<figure class="wp-block-image size-large"><img decoding="async" loading="lazy" src="https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS1-1024x512.jpg" alt="" class="wp-image-14607" srcset="https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS1-1024x512.jpg 1024w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS1-300x150.jpg 300w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS1-768x384.jpg 768w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS1-1536x768.jpg 1536w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS1.jpg 1600w" sizes="(max-width: 1024px) 100vw, 1024px"></figure>
<p>Contextual interference improves the transfer of learning to new situations. There are a few reasons contextual interference works:</p>
<ul>
<li><strong>Identifying problems correctly and ensuring the correct technique is associated with the problem</strong>. A major difficulty in learning isn’t getting knowledge into your head—but getting it out at the right time. Practice that repeats the same technique in narrow situations may result in skills that aren’t accessible when you need them.</li>
<li><strong>Putting similar situations side-by-side helps you compare them</strong>. Seeing two problems that look similar, but require different solution methods, makes it more likely that you’ll attend to the key distinction between them.</li>
<li><strong>The extra effort needed to retrieve the right response may be desirable</strong>. According to psychologist Robert Bjork’s influential <a href="https://www.researchgate.net/profile/Robert-Bjork-2/publication/281322665_A_new_theory_of_disuse_and_an_old_theory_of_stimulus_fluctuation/links/58b6f20945851591c5d55e96/A-new-theory-of-disuse-and-an-old-theory-of-stimulus-fluctuation.pdf">theory of memory</a>, more difficult retrieval results in greater memory strengthening than easier retrieval. Thus, more variable practice is likely more efficient practice.</li>
</ul>
<h2>Abstracting the Deep Structure: Same Idea, Different Examples</h2>
<p>Variability plays a role in abstracting the deep structure of seemingly different situations. Experts tend to <a href="https://www.sciencedirect.com/science/article/abs/pii/S0364021381800298">perceive the deep principles</a> of a particular problem. In contrast, novices tend to get distracted by the superficial features.</p>
<p>Physics experts, for instance, tend to look at problems and see “conservation of energy” or “forces must be balanced if an object isn’t moving.” In contrast, novices tend to look at problems and see “it’s one with a pulley” or “it’s an incline-plane problem.”</p>
<figure class="wp-block-image size-large"><img decoding="async" loading="lazy" src="https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS2-1024x512.jpg" alt="" class="wp-image-14608" srcset="https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS2-1024x512.jpg 1024w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS2-300x150.jpg 300w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS2-768x384.jpg 768w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS2-1536x768.jpg 1536w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS2.jpg 1600w" sizes="(max-width: 1024px) 100vw, 1024px"></figure>
<p>The exact role of concrete versus abstract representations in thinking is controversial in cognitive psychology.</p>
<p>Some theorists argue that we reason through <a href="https://psycnet.apa.org/record/1989-98035-006">storing multiple</a>, <a href="https://psycnet.apa.org/doiLanding?doi=10.1037%2F0033-295X.95.4.492">specific instances of ideas</a>. Others argue that we erase the specifics, leading to <a href="https://en.wikipedia.org/wiki/Prototype_theory">generic stereotypes</a> of situations we deal with. Regardless of whether thinking is fundamentally concrete or abstract (or some mixture of both), seeing multiple examples is central to learning.</p>
<p>A central principle of the highly-successful teaching strategy, <a href="https://en.wikipedia.org/wiki/Direct_instruction">Direct Instruction</a>, is to present students with examples that span the full range of possibilities for a concept. So instead of teaching students to recognize the letter “a” by showing students the exact same letter, we would show “a” in a variety of fonts and typefaces. A student learns that all of these as represent the same “thing” by being exposed to multiple, varying examples:</p>
<figure class="wp-block-image size-large"><img decoding="async" loading="lazy" src="https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS3-1024x512.jpg" alt="" class="wp-image-14605" srcset="https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS3-1024x512.jpg 1024w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS3-300x150.jpg 300w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS3-768x384.jpg 768w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS3-1536x768.jpg 1536w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS3.jpg 1600w" sizes="(max-width: 1024px) 100vw, 1024px"></figure>
<h2>Ensuring Robust Learning: Same Strategy, Varied Problems</h2>
<p>A substantial challenge in learning is that the mind economizes on effort. This means we often fall prey to psychological shortcuts that give us the correct answer, even if they won’t benefit us in future situations.</p>
<p><a href="https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1003425">One study</a> that looked at this required participants to predict a trajectory. Some participants were given a low-variability condition which only involved a few different paths. Other participants were given a high-variability condition which involved a much larger set of different trajectories.</p>
<p>The researchers found that the method participants adopted depended on the variability of the task. When variability was low, participants simply memorized the pattern. In contrast, when variability was high, they simulated the trajectory to find the likely destination.</p>
<p>This result shows that when practice has low variability, we often adopt shortcuts that won’t necessarily generalize to the broader conditions of a task. These effort-saving maneuvers aren’t always conscious, so it’s not simply a matter of laziness. This is one reason I caution students when using flashcards for subjects like mathematics. If you’re not careful, it’s easy to design cards where you memorize the answer (x = 7) and not the method.</p>
<h2>Evolving Practice: Same Problems, Varied Strategies</h2>
<p>Thus far, we’ve discussed variable contexts, examples, or task conditions. But having multiple methods for getting the right answer is also an important part of mastering complex skills.</p>
<p>In contrast to the classical view that experts and novices use a particular method, developmental psychologist Robert Siegler finds that people use <a href="https://www.amazon.ca/Emerging-Minds-Process-Childrens-Thinking-ebook/dp/B000QTD49E/ref=tmm_kin_swatch_0?_encoding=UTF8&amp;qid=&amp;sr=">multiple methods to come up with answers</a>. In his study of how addition strategies children use evolve, he discovered that most children used at least three techniques. Even college students will use multiple methods for single-digit arithmetic.</p>
<p>Following this data, Siegler proposed the “moderate experience hypothesis.” Strategy variability will be highest when we have some, but not a lot, of experience.</p>
<p>Vimla Patel has documented a <a href="https://onlinelibrary.wiley.com/doi/abs/10.1111/j.1365-2923.1991.tb00106.x">similar pattern in the thinking of medical students</a>. Intermediate students, but not novices or experts, show the most elaborations in their reasoning about medical cases.</p>
<p>Siegler argues that this variability is beneficial. First, the “best” strategy depends on the current skill level. Direct retrieval of the correct addition fact (7 + 5 = 12) is easiest—but this strategy only works when we’re confident we know the right answer. Having an assortment of backup methods is helpful in situations when we have lower confidence.</p>
<figure class="wp-block-image size-large"><img decoding="async" loading="lazy" src="https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS5-1024x512.jpg" alt="" class="wp-image-14610" srcset="https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS5-1024x512.jpg 1024w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS5-300x150.jpg 300w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS5-768x384.jpg 768w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS5-1536x768.jpg 1536w, https://www.scotthyoung.com/blog/wp-content/uploads/2022/10/VCS5.jpg 1600w" sizes="(max-width: 1024px) 100vw, 1024px"></figure>
<p>Having multiple strategies for solving a problem is vital when you aren’t yet at the level of mastery. It ensures not only backups you can fall upon when more difficult methods fail, but it gives different reasoning paths to reach the right answer.</p>
<h2>Avoiding Local Optima: Same Challenge, Varied Task Constraints</h2>
<p>A classic learning dilemma is avoiding methods that work well enough, but aren’t the best. In this view of learning, progress is like trying to find the highest point in a vast forest. It makes sense to keep climbing until you reach a peak, but getting stuck on a small hill is easy—even if there’s a taller mountain nearby.</p>
<p>Climbing downhill is hard and feels counterproductive. We instinctively avoid changes that make our performance worse than it might otherwise be, even if we suspect it is good for us in the long run.</p>
<p>One way out of this trap is to practice with variable constraints that prevent using the dominant strategy. A rock climber who leans toward explosive, dynamic jumps might explore a new style by climbing with the constraint of pausing for a second before each new hold. A writer who relies on personal stories might make a rule to avoid them for a future essay.</p>
<h2>When is Variability Good? Watching Out for Cognitive Overload</h2>
<p>Most research supports the benefits of variability in practice. However, less-variable practice is often <a href="https://link.springer.com/article/10.1007/s10648-019-09462-8">better for beginners or lower-performing students</a>. The logic here is relatively straightforward—if you can’t perform a method under helpful, simplified conditions, you probably won’t benefit from making things harder.</p>
<p>Having variable methods may also backfire if some of those methods are buggy or flawed. Researchers have found that <a href="https://www.lri.fr/~mbl/Stanford/CS477/papers/RepairTheory-SeelyBrown.pdf">broken algorithms</a> for basic mathematical procedures lead to difficulty in learning more complex algorithms, such as multi-digit subtraction problems.</p>
<p>These two considerations moderate the extreme stance that all variability is good when learning. Instead, we want to see a slow ramp-up in variability. Learning should start with clear instructions and concrete examples when a skill is new and expand into increasingly broad contexts as we progress.</p>
<h2>Applying Variability to Learning</h2>
<p>Given this research evidence, how can we apply variability to learn better?</p>
<ul>
<li><strong>Mix up your problems</strong>. If you have to study problems, mix them up so there aren’t clues telling you what strategy you need to apply. Mix together Unit One and Unit Two—and don’t indicate which problems come from which sections.</li>
<li><strong>Choose opportunities with greater variability</strong>. Management consultants who work for a wide range of clients are more likely to learn broad principles than those who work only with a single firm or industry. Psychologist Gary Klein <a href="https://www.amazon.com/gp/product/0262532816/ref=ox_sc_saved_title_9?smid=A3DWYIK6Y9EEQB&amp;psc=1">reports</a> that inner-city firefighters progress much faster than rural firefighters, owing to the more varied firefighting conditions. Choosing jobs with greater variability early in your career may lead to better skill development than narrow specializations.</li>
<li><strong>Work with multiple teachers, peers and styles</strong>. Learning a language, for instance, benefits from <a href="https://psycnet.apa.org/record/2013-18820-001">exposure to different speakers</a>, accents and speaking styles. Learning from several teachers is more likely to promote diverse perspectives. Having a variety of peers exposes you to different strategies and is less likely to get you in a rut.</li>
<li><strong>Add constraints that force you away from dominant strategies</strong>. Performing skills under different constraints forces you to explore wider areas of the <a href="https://www.scotthyoung.com/blog/2022/03/28/human-problem-solving/">problem space</a>. This added variability helps you discover new strategies and kicks you out of ruts that can develop when an easy, but sub-optimal, method is relied on.</li>
</ul>
